@@ -45,11 +45,11 @@
 #include <cairo/cairo-xlib.h>
 #include <freerdp/locale/keyboard.h>
 
-static gboolean remmina_rdp_event_on_map(GtkWindow *window, GdkEvent *event, RemminaProtocolWidget *gp)
+gboolean remmina_rdp_event_on_map(RemminaProtocolWidget *gp)
 {
 	TRACE_CALL(__func__);
-	rfContext* rfi = GET_PLUGIN_DATA(gp);
-	rdpGdi* gdi;
+	rfContext *rfi = GET_PLUGIN_DATA(gp);
+	rdpGdi *gdi;
 
 	if (rfi == NULL)
 		return false;
@@ -62,15 +62,23 @@ static gboolean remmina_rdp_event_on_map(GtkWindow *window, GdkEvent *event, Rem
 	return FALSE;
 }
 
-static gboolean remmina_rdp_event_on_unmap(GtkWindow *window, GdkEvent *event, RemminaProtocolWidget *gp)
+gboolean remmina_rdp_event_on_unmap(RemminaProtocolWidget *gp)
 {
 	TRACE_CALL(__func__);
-	rfContext* rfi = GET_PLUGIN_DATA(gp);
-	rdpGdi* gdi;
+	rfContext *rfi = GET_PLUGIN_DATA(gp);
+	rdpGdi *gdi;
 
 	if (rfi == NULL)
 		return false;
 
+	GtkWidget *toplevel = gtk_widget_get_toplevel(GTK_WIDGET(gp));
+	GdkWindow *window = gtk_widget_get_window(toplevel);
+
+	if (gdk_window_get_fullscreen_mode(window) == GDK_FULLSCREEN_ON_ALL_MONITORS) {
+		REMMINA_PLUGIN_DEBUG("Unmap event received, but cannot enable TS_SUPPRESS_OUTPUT_PDU when in fullscreen");
+		return FALSE;
+	}
+
 	gdi = ((rdpContext *)rfi)->gdi;
 
 	REMMINA_PLUGIN_DEBUG("Unmap event received, enabling TS_SUPPRESS_OUTPUT_PDU ");
@@ -82,6 +90,7 @@ static gboolean remmina_rdp_event_on_unmap(GtkWindow *window, GdkEvent *event, R
 static gboolean remmina_rdp_event_on_focus_in(GtkWidget *widget, GdkEventKey *event, RemminaProtocolWidget *gp)
 {
 	TRACE_CALL(__func__);
+
 	rfContext *rfi = GET_PLUGIN_DATA(gp);
 	rdpInput *input;
 	GdkModifierType state;
@@ -93,6 +102,9 @@ static gboolean remmina_rdp_event_on_focus_in(GtkWidget *widget, GdkEventKey *ev
 #endif
 	GdkDevice *keyboard = NULL;
 
+	const gchar *wname = gtk_widget_get_name(gtk_widget_get_toplevel(widget));
+	REMMINA_PLUGIN_DEBUG("Top level name is: %s", wname);
+
 	if (!rfi || !rfi->connected || rfi->is_reconnecting)
 		return FALSE;
 
@@ -192,6 +204,7 @@ static void keypress_list_add(RemminaProtocolWidget *gp, RemminaPluginRdpEvent r
 {
 	TRACE_CALL(__func__);
 	rfContext *rfi = GET_PLUGIN_DATA(gp);
+
 	if (!rdp_event.key_event.key_code)
 		return;
 
@@ -199,7 +212,6 @@ static void keypress_list_add(RemminaProtocolWidget *gp, RemminaPluginRdpEvent r
 		remmina_rdp_event_release_key(gp, rdp_event);
 	else
 		g_array_append_val(rfi->pressed_keys, rdp_event);
-
 }
 
 
@@ -382,6 +394,7 @@ static gboolean remmina_rdp_event_delayed_monitor_layout(RemminaProtocolWidget *
 	gchar *monitorids = NULL;
 	guint32 maxwidth = 0;
 	guint32 maxheight = 0;
+
 	remmina_rdp_monitor_get(rfi, &monitorids, &maxwidth, &maxheight);
 
 	REMMINA_PLUGIN_DEBUG("Sending preconfigured monitor layout");
@@ -403,24 +416,26 @@ static gboolean remmina_rdp_event_delayed_monitor_layout(RemminaProtocolWidget *
 			}
 			rdp_event.type = REMMINA_RDP_EVENT_TYPE_SEND_MONITOR_LAYOUT;
 			if (remmina_plugin_service->file_get_int(remminafile, "multimon", FALSE)) {
-				for (gint i = 0; i < rfi->settings->MonitorCount; ++i) {
+				const rdpMonitor *base = freerdp_settings_get_pointer(rfi->settings, FreeRDP_MonitorDefArray);
+				for (gint i = 0; i < freerdp_settings_get_uint32(rfi->settings, FreeRDP_MonitorCount); ++i) {
+					const rdpMonitor *current = &base[i];
 					REMMINA_PLUGIN_DEBUG("Sending display layout n° %d", i);
-					rdp_event.monitor_layout.Flags = rfi->settings->MonitorDefArray[i].is_primary;
+					rdp_event.monitor_layout.Flags = current->is_primary;
 					REMMINA_PLUGIN_DEBUG("EVNT MON LAYOUT - Flags: %i", rdp_event.monitor_layout.Flags);
-					rdp_event.monitor_layout.Left = rfi->settings->MonitorDefArray[i].x;
+					rdp_event.monitor_layout.Left = current->x;
 					REMMINA_PLUGIN_DEBUG("EVNT MON LAYOUT - Left: %i", rdp_event.monitor_layout.Left);
-					rdp_event.monitor_layout.Top = rfi->settings->MonitorDefArray[i].y;
+					rdp_event.monitor_layout.Top = current->y;
 					REMMINA_PLUGIN_DEBUG("EVNT MON LAYOUT - Top: %i", rdp_event.monitor_layout.Top);
-					rdp_event.monitor_layout.width = rfi->settings->MonitorDefArray[i].width;
+					rdp_event.monitor_layout.width = current->width;
 					REMMINA_PLUGIN_DEBUG("EVNT MON LAYOUT - width: %i", rdp_event.monitor_layout.width);
-					rdp_event.monitor_layout.height = rfi->settings->MonitorDefArray[i].height;
+					rdp_event.monitor_layout.height = current->height;
 					REMMINA_PLUGIN_DEBUG("EVNT MON LAYOUT - height: %i", rdp_event.monitor_layout.height);
-					rdp_event.monitor_layout.physicalWidth = rfi->settings->MonitorDefArray[i].attributes.physicalWidth;
+					rdp_event.monitor_layout.physicalWidth = current->attributes.physicalWidth;
 					REMMINA_PLUGIN_DEBUG("EVNT MON LAYOUT - physicalWidth: %i", rdp_event.monitor_layout.physicalWidth);
-					rdp_event.monitor_layout.physicalHeight = rfi->settings->MonitorDefArray[i].attributes.physicalHeight;
+					rdp_event.monitor_layout.physicalHeight = current->attributes.physicalHeight;
 					REMMINA_PLUGIN_DEBUG("EVNT MON LAYOUT - PhysicalHeight: %i", rdp_event.monitor_layout.physicalHeight);
-					if (rfi->settings->MonitorDefArray[i].attributes.orientation)
-						rdp_event.monitor_layout.desktopOrientation = rfi->settings->MonitorDefArray[i].attributes.orientation;
+					if (current->attributes.orientation)
+						rdp_event.monitor_layout.desktopOrientation = current->attributes.orientation;
 					else
 						rdp_event.monitor_layout.desktopOrientation = rdp_event.monitor_layout.desktopOrientation;
 					REMMINA_PLUGIN_DEBUG("EVNT MON LAYOUT - desktopOrientation: %i", rdp_event.monitor_layout.desktopOrientation);
@@ -430,7 +445,6 @@ static gboolean remmina_rdp_event_delayed_monitor_layout(RemminaProtocolWidget *
 				}
 				remmina_rdp_event_event_push(gp, &rdp_event);
 			} else {
-
 				rdp_event.monitor_layout.width = gpwidth;
 				rdp_event.monitor_layout.height = gpheight;
 				rdp_event.monitor_layout.desktopOrientation = desktopOrientation;
@@ -450,6 +464,7 @@ void remmina_rdp_event_send_delayed_monitor_layout(RemminaProtocolWidget *gp)
 {
 	TRACE_CALL(__func__);
 	rfContext *rfi = GET_PLUGIN_DATA(gp);
+
 	if (!rfi || !rfi->connected || rfi->is_reconnecting)
 		return;
 	if (rfi->delayed_monitor_layout_handler) {
@@ -458,7 +473,6 @@ void remmina_rdp_event_send_delayed_monitor_layout(RemminaProtocolWidget *gp)
 	}
 	if (rfi->scale == REMMINA_PROTOCOL_WIDGET_SCALE_MODE_DYNRES)
 		rfi->delayed_monitor_layout_handler = g_timeout_add(500, (GSourceFunc)remmina_rdp_event_delayed_monitor_layout, gp);
-
 }
 
 static gboolean remmina_rdp_event_on_configure(GtkWidget *widget, GdkEventConfigure *event, RemminaProtocolWidget *gp)
@@ -467,6 +481,7 @@ static gboolean remmina_rdp_event_on_configure(GtkWidget *widget, GdkEventConfig
 	/* Called when gp changes its size or position */
 
 	rfContext *rfi = GET_PLUGIN_DATA(gp);
+
 	if (!rfi || !rfi->connected || rfi->is_reconnecting)
 		return FALSE;
 
@@ -809,6 +824,7 @@ gboolean remmina_rdp_event_on_clipboard(GtkClipboard *gtkClipboard, GdkEvent *ev
 	REMMINA_PLUGIN_DEBUG("owner-change event received");
 
 	rfContext *rfi = GET_PLUGIN_DATA(gp);
+
 	if (rfi)
 		remmina_rdp_clipboard_abort_transfer(rfi);
 
@@ -837,9 +853,20 @@ void remmina_rdp_event_init(RemminaProtocolWidget *gp)
 	GtkClipboard *clipboard;
 	RemminaFile *remminafile;
 
+	gboolean disable_smooth_scrolling = FALSE;
+
 	if (!rfi) return;
 	remminafile = remmina_plugin_service->protocol_plugin_get_file(gp);
 
+	/* we get first the global preferences */
+	s = remmina_plugin_service->pref_get_value("rdp_disable_smooth_scrolling");
+	disable_smooth_scrolling = (s && s[0] == '1' ? TRUE : FALSE);
+	g_free(s), s = NULL;
+	/* Otherwise we use the connection profile specific setting */
+	disable_smooth_scrolling = remmina_plugin_service->file_get_int(remminafile, "disable-smooth-scrolling", disable_smooth_scrolling);
+
+	REMMINA_PLUGIN_DEBUG("Disable smooth scrolling is set to %d", disable_smooth_scrolling);
+
 	rfi->drawing_area = gtk_drawing_area_new();
 	gtk_widget_show(rfi->drawing_area);
 	gtk_container_add(GTK_CONTAINER(gp), rfi->drawing_area);
@@ -847,23 +874,26 @@ void remmina_rdp_event_init(RemminaProtocolWidget *gp)
 	gtk_widget_add_events(rfi->drawing_area, GDK_POINTER_MOTION_MASK
 			      | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
 			      | GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK
-#if GTK_CHECK_VERSION(3, 4, 0)
-			      | GDK_SMOOTH_SCROLL_MASK
-#endif
 			      | GDK_SCROLL_MASK | GDK_FOCUS_CHANGE_MASK);
+
+	if (!disable_smooth_scrolling) {
+		REMMINA_PLUGIN_DEBUG("Adding GDK_SMOOTH_SCROLL_MASK");
+		gtk_widget_add_events(rfi->drawing_area, GDK_SMOOTH_SCROLL_MASK);
+	}
+
 	gtk_widget_set_can_focus(rfi->drawing_area, TRUE);
 
 	remmina_plugin_service->protocol_plugin_register_hostkey(gp, rfi->drawing_area);
 
 	s = remmina_plugin_service->pref_get_value("rdp_use_client_keymap");
 	rfi->use_client_keymap = (s && s[0] == '1' ? TRUE : FALSE);
-	g_free(s);
+	g_free(s), s = NULL;
 
 	/* Read special keymap from profile file, if exists */
 	remmina_rdp_event_init_keymap(rfi, remmina_plugin_service->pref_get_value("rdp_map_keycode"));
 
 	if (rfi->use_client_keymap && rfi->keymap)
-		fprintf(stderr, "RDP profile error: you cannot define both rdp_map_hardware_keycode and have 'Use client keuboard mapping' enabled\n");
+		fprintf(stderr, "RDP profile error: you cannot define both rdp_map_hardware_keycode and have 'Use client keyboard mapping' enabled\n");
 
 	g_signal_connect(G_OBJECT(rfi->drawing_area), "draw",
 			 G_CALLBACK(remmina_rdp_event_on_draw), gp);
@@ -883,10 +913,16 @@ void remmina_rdp_event_init(RemminaProtocolWidget *gp)
 			 G_CALLBACK(remmina_rdp_event_on_key), gp);
 	g_signal_connect(G_OBJECT(rfi->drawing_area), "focus-in-event",
 			 G_CALLBACK(remmina_rdp_event_on_focus_in), gp);
-	g_signal_connect(G_OBJECT(gtk_widget_get_toplevel(rfi->drawing_area)), "map-event",
-			 G_CALLBACK(remmina_rdp_event_on_map), gp);
-	g_signal_connect(G_OBJECT(gtk_widget_get_toplevel(rfi->drawing_area)), "unmap-event",
-			 G_CALLBACK(remmina_rdp_event_on_unmap), gp);
+	/** Fixme: This comment
+	 * needed for TS_SUPPRESS_OUTPUT_PDU
+	 * But it works only when we stay in the same window mode, if we switch to
+	 * fullscreen, for instance, the object refernce is lost, so we loose these
+	 * events.
+	 */
+	//g_signal_connect(G_OBJECT(gtk_widget_get_toplevel(rfi->drawing_area)), "map-event",
+	// G_CALLBACK(remmina_rdp_event_on_map), gp);
+	//g_signal_connect(G_OBJECT(gtk_widget_get_toplevel(rfi->drawing_area)), "unmap-event",
+	// G_CALLBACK(remmina_rdp_event_on_unmap), gp);
 
 	if (!remmina_plugin_service->file_get_int(remminafile, "disableclipboard", FALSE)) {
 		clipboard = gtk_widget_get_clipboard(rfi->drawing_area, GDK_SELECTION_CLIPBOARD);
@@ -1078,6 +1114,7 @@ static void remmina_rdp_event_reconnect_progress(RemminaProtocolWidget *gp, Remm
 {
 	TRACE_CALL(__func__);
 	rfContext *rfi = GET_PLUGIN_DATA(gp);
+
 	gdk_window_invalidate_rect(gtk_widget_get_window(rfi->drawing_area), NULL, TRUE);
 }
 
@@ -1122,6 +1159,7 @@ static BOOL remmina_rdp_event_set_pointer_position(RemminaProtocolWidget *gp, gi
 	TRACE_CALL(__func__);
 	GdkWindow *w, *nw;
 	gint nx, ny, wx, wy;
+
 #if GTK_CHECK_VERSION(3, 20, 0)
 	GdkSeat *seat;
 #else
@@ -1211,6 +1249,7 @@ static void remmina_rdp_ui_event_destroy_cairo_surface(RemminaProtocolWidget *gp
 {
 	TRACE_CALL(__func__);
 	rfContext *rfi = GET_PLUGIN_DATA(gp);
+
 	cairo_surface_destroy(rfi->surface);
 	rfi->surface = NULL;
 }
@@ -1350,6 +1389,7 @@ int remmina_rdp_event_queue_ui_sync_retint(RemminaProtocolWidget *gp, RemminaPlu
 {
 	TRACE_CALL(__func__);
 	int retval;
+
 	ui->sync = TRUE;
 	remmina_rdp_event_queue_ui(gp, ui);
 	retval = ui->retval;
@@ -1361,6 +1401,7 @@ void *remmina_rdp_event_queue_ui_sync_retptr(RemminaProtocolWidget *gp, RemminaP
 {
 	TRACE_CALL(__func__);
 	void *rp;
+
 	ui->sync = TRUE;
 	remmina_rdp_event_queue_ui(gp, ui);
 	rp = ui->retptr;
