@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2009-2011 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2018 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2021 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -151,6 +151,7 @@ struct _RemminaFTPClientPriv {
 	GtkWidget *file_action_widgets[10];
 	gboolean sensitive;
 	gboolean overwrite_all;
+	gboolean resume_all;
 };
 
 static gint remmina_ftp_client_taskid = 1;
@@ -199,10 +200,10 @@ static void remmina_ftp_client_cell_data_filetype_pixbuf(GtkTreeViewColumn *col,
 
 	switch (type) {
 	case REMMINA_FTP_FILE_TYPE_DIR:
-		g_object_set(renderer, "stock-id", "folder", NULL);
+		g_object_set(renderer, "icon-name", "folder", NULL);
 		break;
 	default:
-		g_object_set(renderer, "stock-id", "text-x-generic", NULL);
+		g_object_set(renderer, "icon-name", "text-x-generic", NULL);
 		break;
 	}
 }
@@ -218,17 +219,17 @@ static void remmina_ftp_client_cell_data_progress_pixbuf(GtkTreeViewColumn *col,
 
 	switch (status) {
 	case REMMINA_FTP_TASK_STATUS_WAIT:
-		g_object_set(renderer, "stock-id", "P_ause", NULL);
+		g_object_set(renderer, "icon-name", "media-playback-pause", NULL);
 		break;
 	case REMMINA_FTP_TASK_STATUS_RUN:
-		g_object_set(renderer, "stock-id",
+		g_object_set(renderer, "icon-name",
 			(tasktype == REMMINA_FTP_TASK_TYPE_UPLOAD ? "go-up" : "go-down"), NULL);
 		break;
 	case REMMINA_FTP_TASK_STATUS_FINISH:
-		g_object_set(renderer, "stock-id", "_Yes", NULL);
+		g_object_set(renderer, "icon-name", "emblem-default", NULL);
 		break;
 	case REMMINA_FTP_TASK_STATUS_ERROR:
-		g_object_set(renderer, "stock-id", "_No", NULL);
+		g_object_set(renderer, "icon-name", "emblem-unreadable", NULL);
 		break;
 	}
 }
@@ -237,18 +238,8 @@ static gchar*
 remmina_ftp_client_size_to_str(gfloat size)
 {
 	TRACE_CALL(__func__);
-	gchar *str;
 
-	if (size < 1024.0) {
-		str = g_strdup_printf("%i", (gint)size);
-	}else if (size < 1024.0 * 1024.0) {
-		str = g_strdup_printf("%iK", (gint)(size / 1024.0));
-	}else if (size < 1024.0 * 1024.0 * 1024.0) {
-		str = g_strdup_printf("%.1fM", size / 1024.0 / 1024.0);
-	}else  {
-		str = g_strdup_printf("%.1fG", size / 1024.0 / 1024.0 / 1024.0);
-	}
-	return str;
+	return g_format_size_full((guint64)size, G_FORMAT_SIZE_IEC_UNITS);
 }
 
 static void remmina_ftp_client_cell_data_size(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model,
@@ -275,16 +266,16 @@ static void remmina_ftp_client_cell_data_permission(GtkTreeViewColumn *col, GtkC
 
 	gtk_tree_model_get(model, iter, REMMINA_FTP_FILE_COLUMN_PERMISSION, &permission, -1);
 
-	buf[0] = (permission & 040000 ? 'd' : '-');
-	buf[1] = (permission & 0400 ? 'r' : '-');
-	buf[2] = (permission & 0200 ? 'w' : '-');
-	buf[3] = (permission & 0100 ? 'x' : '-');
-	buf[4] = (permission & 040 ? 'r' : '-');
-	buf[5] = (permission & 020 ? 'w' : '-');
-	buf[6] = (permission & 010 ? 'x' : '-');
-	buf[7] = (permission & 04 ? 'r' : '-');
-	buf[8] = (permission & 02 ? 'w' : '-');
-	buf[9] = (permission & 01 ? 'x' : '-');
+	buf[0] = ((permission & 040000) ? 'd' : '-');
+	buf[1] = ((permission & 0400) ? 'r' : '-');
+	buf[2] = ((permission & 0200) ? 'w' : '-');
+	buf[3] = ((permission & 0100) ? 'x' : '-');
+	buf[4] = ((permission & 040) ? 'r' : '-');
+	buf[5] = ((permission & 020) ? 'w' : '-');
+	buf[6] = ((permission & 010) ? 'x' : '-');
+	buf[7] = ((permission & 04) ? 'r' : '-');
+	buf[8] = ((permission & 02) ? 'w' : '-');
+	buf[9] = ((permission & 01) ? 'x' : '-');
 	buf[10] = '\0';
 
 	g_object_set(renderer, "text", buf, NULL);
@@ -657,14 +648,14 @@ static void remmina_ftp_client_popup_menu(RemminaFTPClient *client, GdkEventButt
 
 	menuitem = gtk_menu_item_new_with_label(_("Download"));
 	gtk_widget_show(menuitem);
-	image = gtk_image_new_from_icon_name("document-save", GTK_ICON_SIZE_MENU);
+	image = gtk_image_new_from_icon_name("remmina-document-save-symbolic", GTK_ICON_SIZE_MENU);
 	gtk_widget_show(image);
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 	g_signal_connect(G_OBJECT(menuitem), "activate", G_CALLBACK(remmina_ftp_client_action_download), client);
 
 	menuitem = gtk_menu_item_new_with_label(_("Upload"));
 	gtk_widget_show(menuitem);
-	image = gtk_image_new_from_icon_name("document-send", GTK_ICON_SIZE_MENU);
+	image = gtk_image_new_from_icon_name("remmina-document-send-symbolic", GTK_ICON_SIZE_MENU);
 	gtk_widget_show(image);
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 	g_signal_connect(G_OBJECT(menuitem), "activate", G_CALLBACK(remmina_ftp_client_action_upload), client);
@@ -748,63 +739,46 @@ static GtkWidget* remmina_ftp_client_create_toolbar(RemminaFTPClient *client)
 	TRACE_CALL(__func__);
 	GtkWidget *box;
 	GtkWidget *button;
-	GtkWidget *image;
 	gint i = 0;
 
 	box = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
-	gtk_widget_show(box);
 	gtk_button_box_set_layout(GTK_BUTTON_BOX(box), GTK_BUTTONBOX_START);
 	gtk_grid_attach(GTK_GRID(client), box, 0, 0, 1, 1);
 
 	button = gtk_button_new_with_label(_("Home"));
-	gtk_widget_show(button);
 	gtk_widget_set_tooltip_text(button, _("Go to home folder"));
-	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
 	gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(remmina_ftp_client_action_home), client);
 
 	button = gtk_button_new_with_label(_("Up"));
-	gtk_widget_show(button);
 	gtk_widget_set_tooltip_text(button, _("Go to parent folder"));
-	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
 	gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(remmina_ftp_client_action_parent), client);
 
 	button = gtk_button_new_with_label(_("Refresh"));
-	gtk_widget_show(button);
 	gtk_widget_set_tooltip_text(button, _("Refresh current folder"));
-	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
 	gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(remmina_ftp_client_action_refresh), client);
 
 	button = gtk_button_new_with_label(_("Download"));
-	gtk_widget_show(button);
 	gtk_widget_set_tooltip_text(button, _("Download from server"));
-	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-	image = gtk_image_new_from_icon_name("document-save", GTK_ICON_SIZE_BUTTON);
-	gtk_button_set_image(GTK_BUTTON(button), image);
 	gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(remmina_ftp_client_action_download), client);
 
 	client->priv->file_action_widgets[i++] = button;
 
 	button = gtk_button_new_with_label(_("Upload"));
-	gtk_widget_show(button);
 	gtk_widget_set_tooltip_text(button, _("Upload to server"));
-	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-	image = gtk_image_new_from_icon_name("document-send", GTK_ICON_SIZE_BUTTON);
-	gtk_button_set_image(GTK_BUTTON(button), image);
 	gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(remmina_ftp_client_action_upload), client);
 
 	button = gtk_button_new_with_label(_("Delete"));
-	gtk_widget_show(button);
 	gtk_widget_set_tooltip_text(button, _("Delete files on server"));
-	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
 	gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(remmina_ftp_client_action_delete), client);
 
 	client->priv->file_action_widgets[i++] = button;
+	gtk_widget_show_all(box);
 
 	return box;
 }
@@ -847,6 +821,21 @@ gboolean remmina_ftp_client_get_overwrite_status(RemminaFTPClient *client)
 	return client->priv->overwrite_all;
 }
 
+/* Set the resume status */
+void remmina_ftp_client_set_resume_status(RemminaFTPClient *client, gboolean status)
+{
+	TRACE_CALL(__func__);
+	client->priv->resume_all = status;
+}
+
+/* Get the resume status */
+gboolean remmina_ftp_client_get_resume_status(RemminaFTPClient *client)
+{
+	TRACE_CALL(__func__);
+	return client->priv->resume_all;
+}
+
+
 static void remmina_ftp_client_init(RemminaFTPClient *client)
 {
 	TRACE_CALL(__func__);
@@ -864,6 +853,8 @@ static void remmina_ftp_client_init(RemminaFTPClient *client)
 
 	/* Initialize overwrite status to FALSE */
 	client->priv->overwrite_all = FALSE;
+	/* Initialize resume status to FALSE */
+	client->priv->resume_all = FALSE;
 
 	/* Main container */
 	gtk_widget_set_vexpand(GTK_WIDGET(client), TRUE);
@@ -871,6 +862,7 @@ static void remmina_ftp_client_init(RemminaFTPClient *client)
 
 	/* Toolbar */
 	toolbar = remmina_ftp_client_create_toolbar(client);
+	gtk_widget_set_hexpand(toolbar, TRUE);
 
 	/* The Paned to separate File List and Task List */
 	vpaned = gtk_paned_new(GTK_ORIENTATION_VERTICAL);
@@ -910,7 +902,7 @@ static void remmina_ftp_client_init(RemminaFTPClient *client)
 
 	/* Remote File List - Columns */
 	column = gtk_tree_view_column_new();
-	gtk_tree_view_column_set_title(column, _("File Name"));
+	gtk_tree_view_column_set_title(column, _("Filename"));
 	gtk_tree_view_column_set_expand(column, TRUE);
 	gtk_tree_view_column_set_resizable(column, TRUE);
 	gtk_tree_view_column_set_sort_column_id(column, REMMINA_FTP_FILE_COLUMN_NAME_SORT);
@@ -979,7 +971,7 @@ static void remmina_ftp_client_init(RemminaFTPClient *client)
 
 	/* Task List - Columns */
 	column = gtk_tree_view_column_new();
-	gtk_tree_view_column_set_title(column, _("File Name"));
+	gtk_tree_view_column_set_title(column, _("Filename"));
 	gtk_tree_view_column_set_resizable(column, TRUE);
 	gtk_tree_view_column_set_expand(column, TRUE);
 	gtk_tree_view_column_set_sort_column_id(column, REMMINA_FTP_TASK_COLUMN_NAME);
@@ -1023,7 +1015,7 @@ static void remmina_ftp_client_init(RemminaFTPClient *client)
 
 	renderer = remmina_cell_renderer_pixbuf_new();
 	column = gtk_tree_view_column_new_with_attributes(NULL, renderer, NULL);
-	g_object_set(G_OBJECT(renderer), "stock-id", "_Cancel", NULL);
+	g_object_set(G_OBJECT(renderer), "icon-name", "process-stop", NULL);
 	gtk_tree_view_column_set_resizable(column, FALSE);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(priv->task_list_view), column);
 
